from __future__ import annotations

import re

import discord
from aiohttp import ClientSession
from discord.ext import commands


class Night(commands.Cog):
    @commands.hybrid_command()
    @commands.is_owner()
    async def reload(self, ctx: commands.Context):
        print("reload")
        bot: commands.Bot = ctx.bot
        try:
            await bot.reload_extension("nightly.night")
        except commands.ExtensionNotLoaded:
            await ctx.reply("not loaded")
        print("reloaded")
        await ctx.reply("reloaded")

    @commands.hybrid_command()
    @commands.is_owner()
    async def sync(self, ctx: commands.Context):
        await ctx.bot.tree.sync()
        print("synced")
        await ctx.reply("synced")

    @commands.hybrid_command()
    async def leetcode(self, ctx: commands.Context, url: str):
        match = re.search(r"/problems/([a-z0-1\-]*)", url)
        if not match:
            await ctx.reply("invalid url")
            return
        name = match.group(1)
        async with ClientSession() as session:
            async with session.post(
                "https://leetcode.com/graphql/",
                json={
                    "query": "\n    query consolePanelConfig($titleSlug: String!) {\n  question(titleSlug: $titleSlug) {\n    questionId\n    questionFrontendId\n    questionTitle\n    enableDebugger\n    enableRunCode\n    enableSubmit\n    enableTestMode\n    exampleTestcaseList\n    metaData\n  }\n}\n    ",
                    "variables": {"titleSlug": name},
                    "operationName": "consolePanelConfig",
                },
            ) as response:
                if not response.ok:
                    await ctx.reply("error (idk)")
                    return
                match await response.json():
                    case {"data": {"question": {"questionFrontendId": qfi, "questionTitle": qt}}}:
                        title = f"{qfi}. {qt}"
                        print(title)
                    case _:
                        await ctx.reply("error (leetcode format wrong)")
                        return
                if not isinstance(ctx.channel, discord.TextChannel):
                    await ctx.reply("not available outside text channels")
                    return
                thread = await ctx.channel.create_thread(name=title, type=discord.ChannelType.public_thread)
                message = await thread.send(f"https://leetcode.com/problems/{name}/")
                if not ctx.bot_permissions.manage_messages:
                    await ctx.reply("cannot pin messages (missing permissions)", mention_author=False)
                    return
                await message.pin()


async def setup(bot: commands.Bot):
    global cog
    cog = Night()
    await bot.add_cog(cog)


async def teardown(bot: commands.Bot):
    global cog
    await bot.remove_cog(cog.qualified_name)
    del cog
    print("torn down")
